var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var IoC;
(function (IoC) {
    function Resource(config) {
        if (typeof config === "function") {
            return ResourceDecorator(null, config);
        }
        return function (target) {
            return ResourceDecorator(config, target);
        };
    }
    IoC.Resource = Resource;
    function ResourceDecorator(config, target) {
        var existingInjectedParameters = Reflect.getOwnMetadata("params_inject", target) || [];
        var newConstructor;
        if (existingInjectedParameters.length > 0) {
            var paramTypes_1 = Reflect.getMetadata("design:paramtypes", target);
            newConstructor = InjectorHanlder.decorateConstructor(function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i - 0] = arguments[_i];
                }
                var newArgs = args ? args.concat() : new Array();
                for (var _a = 0, existingInjectedParameters_1 = existingInjectedParameters; _a < existingInjectedParameters_1.length; _a++) {
                    var index = existingInjectedParameters_1[_a];
                    if (index >= newArgs.length) {
                        newArgs.push(Container.get(paramTypes_1[index]));
                    }
                }
                target.apply(this, newArgs);
                Container.applyInjections(this, target);
            }, target);
        }
        else {
            newConstructor = InjectorHanlder.decorateConstructor(function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i - 0] = arguments[_i];
                }
                target.apply(this, args);
                Container.applyInjections(this, target);
            }, target);
        }
        if (config) {
            var Config_1 = Container.bind(newConstructor);
            if ('provider' in config) {
                Config_1.provider(config.provider);
            }
            else if ('bindTypeName' in config) {
                Config_1.toTypeNamed(config.bindTypeName);
            }
            else {
                Config_1.to(newConstructor);
            }
            if (config.scope) {
                Config_1.scope(config.scope);
            }
        }
        return newConstructor;
    }
    function Inject() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i - 0] = arguments[_i];
        }
        if (args.length == 2) {
            return InjectPropertyDecorator.apply(this, args);
        }
        else if (args.length == 3 && typeof args[2] === "number") {
            return InjectParamDecorator.apply(this, args);
        }
        throw new Error("Invalid @Inject Decorator declaration.");
    }
    IoC.Inject = Inject;
    function InjectPropertyDecorator(target, key) {
        var t = Reflect.getMetadata("design:type", target, key);
        Container.addPropertyInjector(target.constructor, key, t);
    }
    function InjectParamDecorator(target, propertyKey, parameterIndex) {
        if (!propertyKey) {
            var existingInjectedParameters = Reflect.getOwnMetadata("params_inject", target, propertyKey) || [];
            existingInjectedParameters.push(parameterIndex);
            Reflect.defineMetadata("params_inject", existingInjectedParameters, target, propertyKey);
        }
    }
    var Map = (function () {
        function Map() {
        }
        Map.prototype.put = function (key, value) {
            this[key] = value;
        };
        Map.prototype.get = function (key) {
            return this[key];
        };
        return Map;
    }());
    var Container = (function () {
        function Container() {
        }
        Container.bind = function (source) {
            checkType(source);
            var config = Container.bindings.get(source);
            if (!config) {
                config = new ConfigImpl(source);
                Container.bindings.put(source, config);
            }
            return config;
        };
        Container.get = function (source) {
            checkType(source);
            var config = Container.bindings.get(source);
            if (!config) {
                config = this.bind(source).to(source);
            }
            return config.getInstance();
        };
        Container.applyInjections = function (toInject, targetType) {
            if (targetType) {
                var injections = InjectorHanlder.getInjectorFromType(targetType, false);
                injections.forEach(function (entry) {
                    entry(toInject);
                });
            }
            else {
                var injections = InjectorHanlder.getInjectorFromInstance(toInject);
                injections.forEach(function (entry) {
                    entry(toInject);
                });
            }
        };
        Container.addPropertyInjector = function (target, key, propertyType) {
            var injections = InjectorHanlder.getInjectorFromType(target, false);
            injections.push(function (toInject) {
                Container.injectProperty(toInject, key, propertyType);
            });
        };
        Container.injectProperty = function (toInject, key, source) {
            toInject[key] = Container.get(source);
        };
        Container.bindings = new Map();
        return Container;
    }());
    IoC.Container = Container;
    function checkType(source) {
        if (!source) {
            throw new TypeError('Invalid type requested to IoC ' +
                'container. Type is not defined.');
        }
    }
    var Config = (function () {
        function Config() {
        }
        return Config;
    }());
    IoC.Config = Config;
    var ConfigImpl = (function (_super) {
        __extends(ConfigImpl, _super);
        function ConfigImpl(source) {
            _super.call(this);
            this.source = source;
        }
        ConfigImpl.prototype.to = function (target) {
            checkType(target);
            this.iocprovider = function () {
                return new target();
            };
            return this;
        };
        ConfigImpl.prototype.toTypeNamed = function (target) {
            this.iocprovider = function () {
                checkType(window[target]);
                var c = Object.create(window[target].prototype);
                var targetType = c.constructor;
                checkType(targetType);
                return new targetType();
            };
            return this;
        };
        ConfigImpl.prototype.provider = function (provider) {
            this.iocprovider = provider;
            return this;
        };
        ConfigImpl.prototype.scope = function (scope) {
            this.iocscope = scope;
            return this;
        };
        ConfigImpl.prototype.getInstance = function () {
            if (!this.iocscope) {
                this.scope(Scope.Local);
            }
            return this.iocscope.resolve(this.iocprovider, this.source);
        };
        return ConfigImpl;
    }(Config));
    IoC.ConfigImpl = ConfigImpl;
    var Scope = (function () {
        function Scope() {
        }
        return Scope;
    }());
    IoC.Scope = Scope;
    var LocalScope = (function (_super) {
        __extends(LocalScope, _super);
        function LocalScope() {
            _super.apply(this, arguments);
        }
        LocalScope.prototype.resolve = function (provider, source) {
            return provider();
        };
        return LocalScope;
    }(Scope));
    IoC.LocalScope = LocalScope;
    Scope.Local = new LocalScope();
    var SingletonScope = (function (_super) {
        __extends(SingletonScope, _super);
        function SingletonScope() {
            _super.apply(this, arguments);
        }
        SingletonScope.prototype.resolve = function (provider, source) {
            var instance = SingletonScope.instances.get(source);
            if (!instance) {
                instance = provider();
                SingletonScope.instances.put(source, instance);
            }
            return instance;
        };
        SingletonScope.instances = new Map();
        return SingletonScope;
    }(Scope));
    IoC.SingletonScope = SingletonScope;
    Scope.Singleton = new SingletonScope();
    var InjectorHanlder = (function () {
        function InjectorHanlder() {
        }
        InjectorHanlder.decorateConstructor = function (derived, base) {
            for (var p in base) {
                if (base.hasOwnProperty(p) && !derived.hasOwnProperty(p)) {
                    derived[p] = base[p];
                }
            }
            derived['__parent'] = base;
            function __() { this.constructor = derived; }
            derived.prototype = base === null ? Object.create(base) :
                (__.prototype = base.prototype, new __());
            return derived;
        };
        InjectorHanlder.getConstructorFromType = function (target) {
            var typeConstructor = target;
            if (typeConstructor['name']) {
                return typeConstructor;
            }
            while (typeConstructor = typeConstructor['__parent']) {
                if (typeConstructor['name']) {
                    return typeConstructor;
                }
            }
            throw TypeError('Can not identify the base Type for requested target');
        };
        InjectorHanlder.getConstructorFromInstance = function (target) {
            return InjectorHanlder.getConstructorFromType(target.constructor);
        };
        InjectorHanlder.getInjectorFromType = function (target, recursive) {
            var baseConstructor = InjectorHanlder.getConstructorFromType(target);
            if (!InjectorHanlder.typeInjections[baseConstructor]) {
                InjectorHanlder.typeInjections[baseConstructor] = new Array();
            }
            var result = InjectorHanlder.typeInjections[baseConstructor];
            if (recursive) {
                var parent_1 = baseConstructor['__parent'];
                if (parent_1) {
                    result = InjectorHanlder.getInjectorFromType(parent_1, true).concat(result);
                }
            }
            return result;
        };
        InjectorHanlder.getInjectorFromInstance = function (target) {
            var baseConstructor = InjectorHanlder.getConstructorFromInstance(target);
            return InjectorHanlder.getInjectorFromType(baseConstructor, true);
        };
        InjectorHanlder.typeInjections = new Map();
        return InjectorHanlder;
    }());
})(IoC || (IoC = {}));
//# sourceMappingURL=IoC.js.map